#
# Licensed Materials - Property of IBM
# 5765-FS1
# (c) Copyright IBM Corp. 2003  All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# OBDC_Common.pm --
#    Common variables and routines used by all of the other PMF packages.
#

package OBDC_Common;

use strict;

#
# Define thread data needed to do asynchronous execution on Windows.  Note that
# we must use the require/import instead of just a use statement.  A use statement
# would always get executed regardless of the check for the Windows platform.
#
BEGIN {
    if ( $^O eq "MSWin32") {
	require threads;
	import threads;
	require threads::shared;
	import threads::shared;
	our @threadStatus : shared;
    }
}

our (%dataCollectValues, %roleValues);
our ($collectAny, $collectBasic, $collectFull, $collectInternal, $collectOSDump,
     $collectPerf);
our ($roleAny, $roleClient, $roleServer, $roleAdmin);

my $asyncWaitInterval = 3;

%dataCollectValues = (
		      "basic" => 0x01,
		      "full" => 0x02,
		      "internal" => 0x04,
		      "osdump" => 0x08,
		      "perf" => 0x10
		      );

%roleValues = (
	       "client" => 0x01,
	       "server" => 0x02,
	       "admin" => 0x04
	       );

#
# The $collect*, and the $role* variables are just for convenience.  To
# make accessing the %dataCollectValues and %roleValues easier.
#
$collectAny = 0xFFFF;
$collectBasic = $dataCollectValues{"basic"};
$collectFull = $dataCollectValues{"full"};
$collectInternal = $dataCollectValues{"internal"};
$collectOSDump = $dataCollectValues{"osdump"};
$collectPerf = $dataCollectValues{"perf"};

$roleAny = 0xFFFF;
$roleClient = $roleValues{"client"};
$roleServer = $roleValues{"server"};
$roleAdmin = $roleValues{"admin"};

#
#  ExecuteShellCommands() --
#    Accept a string that is a comma separated list of shell style commands.
#    Execute each command in the comma separated list.  Note that we are not
#    concerned if the command fails.  All output from the command will go to 
#    the STDOUT. (See code in pmf.pl that resets STDOUT.)
#
#    Arguments:
#        commands -- comma separated string of shell commands
#
sub ExecuteShellCommands {
    my (@commands, $cmd) = split(/,/, $_[0]);

    foreach $cmd (@commands) {
	print "\nExecuting => $cmd\n\n";
	system($cmd);
	print "\n";
    }
}

#
# FormatTimeData --
#    Does some simple time formatting to make displaying time information easier.
#
#    Arguments:
#      timeVal -- the time in seconds to convert to a string.  If this is zero then
#                 convert the current time.
#      the rest -- are the same as the arguments returned by localtime()
#
sub FormatTimeData {
    my ($timeVal, $seconds, $minutes, $hour, $monthDay, $month, $year, $weekDay,
	$yearDay, $isdst);

    $timeVal = ($_[0] == 0) ? time() : $_[0];

    ($seconds,$minutes,$hour,$monthDay,$month,$year,$weekDay,$yearDay,$isdst) = localtime($timeVal);
    $year = $year + 1900; # The year is from a base of 1900.
    $month = $month + 1;  # Months are indexed from zero; we want to start with one
    $month = ($month < 10) ? "0$month" : $month;
    $monthDay = ($monthDay < 10) ? "0$monthDay" : $monthDay;
    $hour = ($hour < 10) ? "0$hour" : $hour;
    $minutes = ($minutes < 10) ? "0$minutes" : $minutes;
    $seconds = ($seconds < 10) ? "0$seconds" : $seconds;

    return($seconds, $minutes, $hour, $monthDay, $month, $year, $weekDay, $yearDay,
	   $isdst);
}

#
# ComputeMaxIterations() --
#    Given the number of seconds to wait, this calculates the number of loop
#    iterations of $asyncWaitInterval that must be done to reach the maximum
#    seconds provided as an argument.
#
#    Args:
#      secsToWait -- the maximum number of seconds to wait
#
#    Returns:  the maximum number of iterations
#
sub ComputeMaxIterations {
    my $secsToWait = $_[0];
    my ($iterations);

    $secsToWait += ($iterations = $secsToWait % $asyncWaitInterval) 
	? $asyncWaitInterval - $iterations : 0; 
    return($secsToWait / $asyncWaitInterval);
}

#
# DangerousUnixCmd() --
#    Routine that will issue a Unix command and wait for it to complete for some 
#    time.  If it does not complete, it is killed.  This is used to avoid invoking
#    commands that may hang.  Generally these are ones involving some SAN FS
#    component.
#
#    Arguments:
#      cmdName -- name of the command to execute
#      cmdArgs -- arguments to the command
#      outputDir -- directory where output can be placed (for a temp file)
#      secsToWait -- time to wait in seconds
#
sub DangerousUnixCmd {
    my ($cmdName, $cmdArgs, $tempFile, $secsToWait) = @_;
    my ($iterations, $maxIterations, $cmdPid, $cmdTty);

    $tempFile .= "/pmftemp.$$";
    $cmdTty = `tty | sed -e 's:/dev/::'`;  # Needed for ps command 
    chop $cmdTty;
    system("$cmdName $cmdArgs > $tempFile &");  # Execute the command
    sleep 2;  # Give the command a chance to run.
    $_ = `ps -ft $cmdTty | fgrep $cmdName | fgrep -v fgrep`;
    if ($_) {
	/(\d+)\s.*/;
	$cmdPid = $1;  # Capture process id to check on process existence.

	$secsToWait += ($iterations = $secsToWait % $asyncWaitInterval) 
	    ? $asyncWaitInterval - $iterations : 0; 
	$maxIterations = $secsToWait / $asyncWaitInterval;

	for ($iterations = 0; $iterations < $maxIterations; $iterations++) { 
	    #
	    # Sending signal zero is a cheap way to check to see if the process
	    # is still running.
	    #
	    if (!kill(0, $cmdPid)) {
		system("cat $tempFile");  # Capture the output in our file.
		unlink("$tempFile");
		return;
	    }
	    sleep $asyncWaitInterval;
	}
	print "*** WARNING *** -- Command \"$cmdName\" failed to complete.\n";
        kill(9, $cmdPid);
    }
    else {
	system("cat $tempFile");  # Capture the output in our file.
	unlink("$tempFile");
    }
}

#
#  ExecuteDangerousShellCmds() --
#    Asynchronously execute shell commands that might hang and inhibit the OBDC from
#    completing.
#
#    Arguments:
#      cmdName -- name of the command to execute
#      cmdArgs -- arguments to the command
#      outputDir -- directory where output can be placed (for a temp file)
#      secsToWait -- time to wait in seconds
#
sub ExecuteDangerousShellCmds {
    print "\nExecuting => $_[0] $_[1]\n\n";
    if ( $^O eq "MSWin32") {
	print ">>> ExecuteDangerousShellCmds NOT YET IMPLEMNTED FOR WINDOWS <<<\n";
    }
    else {
	&DangerousUnixCmd(@_);
    }
    print "\n";
}

#
# DangerousWindowsFunction() --
#    Routine that will execute a function on Windows in a separate thread and wait 
#    some time for it to complete.  If it does not complete, then the this function
#    simply returns.  It does not kill the thread because the Perl thread interface
#    does not currently support this capability. 
#
#    Arguments:
#      funcName -- name of the function to execute
#      outputDir -- directory where output can be placed (for a temp file)
#      secsToWait -- time to wait in seconds
#
sub DangerousWindowsFunction {
    my ($funcName, $tempFile, $secsToWait) = @_;
    my ($iterations, $maxIterations, $funcThread, $threadId);

    $maxIterations = &ComputeMaxIterations($secsToWait);

    #
    #  Since the functions we invoke via this path are not aware that they are being
    #  executed asynchronously, we need to reset STDOUT/STDERR to a temporary file
    #  in case they hang.
    #
    open(SAVESTDOUT, ">&STDOUT");
    open(SAVESTDERR, ">&STDERR");

    $tempFile .= "\\pmftemp.$$";
    if (open(STDOUT, ">$tempFile")) {
	open(STDERR, ">&STDOUT");
    }
    else {
	print "FAILED to redirect output for dangerous function $funcName.\n";
	return;
    }

    $funcThread = threads->create(\&$funcName, 0);
    $threadId = $funcThread->tid;

    #
    #  Wait for the thread to complete.
    #
    for ($iterations = 0; $iterations < $maxIterations; $iterations++) { 
	#
	#  If the value of threadStatus becomes non-zero then the thread has
	#  completed its work.
	#
	if ($OBDC_Common::threadStatus[$threadId]) {
	    $funcThread->join;
	    #
	    # Restore STDOUT and STDERR.
	    #
	    close(STDOUT);
	    close(STDERR);
	    open(STDOUT, ">&SAVESTDOUT");
	    open(STDERR, ">&SAVESTDERR");
	    system("type \"$tempFile\"");  # Capture the output in our file.
	    unlink("$tempFile");
	    return;
	}
	sleep $asyncWaitInterval;	
    }

    print "*** WARNING *** -- Function \"$funcName\" failed to complete.\n";

    #
    #  The Perl threading API does not include a method for asynchronously killing a
    #  thread, so we just let it run/hang till this script exits.  At this point we
    #  detach the thread to let Perl know we don't care about it, though I'm not
    #  certain this is really needed.
    #  NOTE:  I'm uncertain how changing STDOUT/STDERR will impact the hung thread
    #         should it un-hang itself prior to this script completing.
    #
    $funcThread->detach();
    close(STDOUT);
    close(STDERR);
    open(STDOUT, ">&SAVESTDOUT");
    open(STDERR, ">&SAVESTDERR");
}

#
#  ExecuteDangerousFunction() --
#    Asynchronously execute a function that might hang and inhibit the OBDC from
#    completing.
#
#    Arguments:
#      funcName -- name of the function to execute
#      outputDir -- directory where output can be placed (for a temp file)
#      secsToWait -- time to wait in seconds
sub ExecuteDangerousFunction {
    print "\nExecuting => $_[0] \n\n"; # CHECK -- DO YOU NEED THIS?
    if ( $^O eq "MSWin32") {
	&DangerousWindowsFunction(@_);
    }
    else {
	print ">>> ExecuteDangerousFunction NOT YET IMPLEMNTED FOR UNIX <<<\n";
    }
    print "\n";

}
