#
#  Licensed Materials - Property of IBM
#  5765-FS1
#  (c) Copyright IBM Corp. 2003  All Rights Reserved
#  US Government Users Restricted Rights - Use, duplication or 
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#  pmf.pl --
#    This script is used to gather system and IBM SMI-S TapeLibrary Agent
#    information to aid in resovling customer reported problems.  It is 
#    expected that this script will be run in response to customer problems
#    that have no easy first answer.  The script gathers all of its data
#    and places it in a directory from where it can be viewed or sent back
#    to IBM
#
#    For usage information see the Usage function.
#

package Pmf_Main;

use strict;
use Getopt::Long;
use OBDC_Common;

require "ctime.pl";

#
#  Background --
#    The organization of this script is data driven via the data structures
#    defined below.  The intent for using a data driven approach was to help
#    organize the commands that are executed by this script to obtain the 
#    necessary data. This organization should help to simplify modifications of
#    the script to gather new data or support new platforoms.  Such changes
#    should only require modifying a single function or extending the
#    existing data structures.  The core logic will hopefully remain unchanged.
#
#    Each platform upon which this script runs has a file that contains its
#    specific data structures and functions.  These files are named as,
#    Pmf_<platform>_Data.pm, where <platform> is AIX, Linux, Windows, etc.
#    This had to be done to maintain this common top level script.  In testing
#    even though this script protected "use" commands for Windows specific
#    pacakges with checks to see that it was running on Windows, the "use"
#    commands were always executed by Perl.  The data files are defined as 
#    packages so that this script can properly access the variables and 
#    functions in those files.  The names of the data structures and 
#    functions are the same in each of the data files.  This 
#    does not cause any name conflicts since only one of the data file
#    packages will be included in this script at any given time.  Using the 
#    the names across all the data file packages simplifies some of the logic
#    in this script, and should simplify maintenance.
#

my ($myname, $myversion,
    $platform, $home, $topOutputDir, $collectionOption, $dataFile, $tankRoleStr,
    $tankRoleBits, $optsResult, $delimiterString, $printHelp, $outputDir, 
    $archiveName, %commonData, $pathSeparator, $archiveData, @machineName,
    $archiveFunction, $perfOption, $saveOSDump, $extraOptions, $collectOptions, 
    $collectDataBits, $collectDataStr, $dataOption, @badCollectOptions,
    $pmfLogFile);

$myname   = "obdc";
$myversion = "1.0.0";
$platform = $^O;
$archiveData = 1;
$collectionOption = "f";
$tankRoleStr = "";  # Is tank running as a client or server.
$dataFile = "diagnostics.txt";  # default output filename
$delimiterString = "Component: ";
$printHelp = 0;
$archiveName = "IBM-TapeLib-SMISAgent-OBDC-Dump";
$collectOptions = "basic,full,internal,osdump,perf";

#
# Data Structure Defintion --
#
# The data structures are used to execute the processing needed to collect
# the appropriate data.  There are two data structures used.  They are described here
# so that this information does not need to be duplicated in each 
# Pmf_<platform>_Data.pm file.  See those files for the actual use of the structures.
#
# The first structure is used to define the components/subcomponents of whcih data
# will be gathered.  It has the following fields.
#
#  colopt -- defines under which collection option (basic, full, etc.) that this
#            set of data is to be collected
#  roleopt -- defines under which Newport role (client, server) that this
#             set of data is to be collected
#  functions -- can contain a pointer a functions structure (see below) or this 
#               field can be undefined.
#  subcomps -- can contain a pointer to another component structure that defines the
#              subcomponents that are part of this component.  It can also be 
#              undefined.  Note, this field allows nesting to any level.
#
# This next structure is used to define all of the functions to invoke to gather 
# data for a component/subcomponent.  It has the following fields.
#
#  colopt -- defines under which collection option (basic, full, etc.) that this
#            set of data is to be collected
#  func -- a pointer to the function to invoke; a function can be defined for the 
#          following values.
#
#           info -- general information (name, version, configuration)
#           status -- current status
#           diags -- generate diagnostics
#           perf -- generate performance information
#           logs -- gather log files
#
#

#
#  Output formatting functions.
#
sub PrintStartSection {
    print "\nSTART Section -- @_ Information\n";
}

sub PrintEndSection {
    print "\nEND Section -- @_ Information\n";
}

sub PrintStartDelimiter {
    print "\nSTART -- $delimiterString\n";
}

sub PrintEndDelimiter {
    print "\nEND -- $delimiterString\n";
}

sub PrintDataMarker {
    print "\nDATA -- @_\n";
}

#
#  ErrorExit -- print an error message and exits with a non-zero value
#
#    Arguments:
#        errorMessage -- error message text to print
#
sub ErrorExit {
    my ($errorMessage) = @_;
    print "[ERROR]: $errorMessage\n";
    exit 1;
}

#
# UnrecognizedCmdLineArg() --
#    A simple function that will be invoked if the command line processing sees
#    something it does not recognize.  It was added to provide better feedback to 
#    users that invoke the script with extraneous or bad arguments.
#
#    Arguments:
#      badValue -- the name of the bad command line value
#
sub UnrecognizedCmdLineArg {
    print "The argument, $_[0], is invalid and will be ignored.\n";
}

#
# Usage -- print the help information for this script.
#
sub Usage {
    my ($programName);

    $programName = ($platform eq "MSWin32") ? "$myname.bat" : "$myname";

    print "Usage: $programName\n";
    print "       -home <directory> - home directory of Storage Tank\n";
    print "                           Default: /opt/IBM/smis/tapeagent\n";
    print "       -help - print this message\n";
}

#
#  RetrievePmfData() --
#    This function returns a hash of various data that is needed by the platform
#    specific functions in the Pmf_<platform_Data.pm modules.  Yes, this is
#    kind of ugly but it was the only way to pull this off.  You could create
#    a sort of Pmf_Common.pm that might do this but that did not seem much
#    better than this.  Note that this function assumes it is called after
#    all initialization is done by this script.
#
sub RetrievePmfData {
    %commonData = ("home" => $home,
		   "outputDir" => $outputDir,
		   "serverOutputDir" => "${outputDir}",
		   "archiveFile" => $archiveName);
    return %commonData;
}

#
# CheckCollectDataOptions () --
#    A simple function for platform specific functions to invoke to determine
#    if a certain data collection option has been enabled by the user.
#
#    Arguments --
#        $_ --  a bitmask of the options to check.
#
sub CheckCollectDataOptions {
    return (($_[0] & $collectDataBits) ? 1 : 0);
}

#
# ExecuteDataFunctions() --
#    This function will walk through all the functions for a given 
#    component or sub-component and execute them.  
#
#    Arguments:
#        funcsPtr -- pointer to a hash of data functions
#
sub ExecuteDataFunctions {
    my ($funcsPtr) = $_[0];
    my ($fname, %hashedFuncs);

    %hashedFuncs = %$funcsPtr;

    #
    # Invoke all of the functions defined for this component.
    #
    foreach $fname (keys %hashedFuncs) {
	if ($hashedFuncs{$fname}{"colopt"} & $collectDataBits) {
	    PrintDataMarker($fname);
	    $hashedFuncs{$fname}{"func"}();
	}
    }
}

#
# GatherData() -- 
#    Collect data for one of the major areas of information gathered by this
#    script.  Currently there are two, hardware and software.
#
#    This function is the core of this script.  It recursively walks through
#    the data structures.  It prints any data delimiters to mark what data
#    is being collected.  It does this by concatenating appropriate strings
#    to mark the data, and will subsequently remove those strings prior to
#    returning.
#
#    Arguments:
#        hashPtr -- a reference to a hash of all the componets and the 
#                   metadata/functions used to determine if we should include
#                   there data in the output
#
sub GatherData {
    my ($hashPtr) = $_[0];
    my (@compNames, %hashData, $compName, $compInfo, $subcompsPtr, %subcomps,
	@subcompsNames);

    %hashData = %$hashPtr;

    foreach $compName (keys %hashData) {
	$compInfo = $hashData{$compName};

	#
	#  Check to see if this {sub}component is to have its functions called
	#  based on the collection option and the role this machine plays.
	#
	if (($compInfo->{colopt} & $collectDataBits) &&
	    ($compInfo->{roleopt} & $tankRoleBits)) {
	    $delimiterString .= $compName;   # Add the component to delimiter
	    &PrintStartDelimiter;
	    if ($compInfo->{functions}) {
		&ExecuteDataFunctions($compInfo->{functions});
	    }
	    #
	    # If there are sub-components for this component then work through
	    # that list.
	    #
	    if ($compInfo->{subcomps}) {
		$subcompsPtr = $compInfo->{subcomps};
		%subcomps = %$subcompsPtr;
		$delimiterString = "Sub" . $delimiterString . "->";
		&GatherData(\%subcomps);
		$delimiterString =~ s/^Sub(.*)->$/$1/;
	    }

	    &PrintEndDelimiter;
	    $delimiterString =~ s/$compName$//;
	}
    }
}

#
# Main
#
# Parse the command line.
#
$optsResult = GetOptions("home=s" => \$home,
			 "help" => \$printHelp,
			 '<>' => \&UnrecognizedCmdLineArg);

if ($printHelp) {
    &Usage;
    exit 0;
}

if ($collectDataStr eq "") {
    $collectDataStr = "full";
}
else {
    $collectDataStr =~ s/ +//g;  # TODO -- IS THIS NEEDED???
}

#
# Validate that any collection options provided are legitimate.  Warn the user of
# any that are not and remove them from the list.  Then check to see if there are
# any remaining.
#
my ($foundBasic, $foundFull);

foreach $dataOption (split(/,/, $collectDataStr)) {
    if (!($collectOptions =~ /$dataOption/)) {
	print "Collection option, $dataOption, is not valid and will be ignored.\n";
	push(@badCollectOptions, $dataOption);
    }
    else {
	$collectDataBits |= $OBDC_Common::dataCollectValues{$dataOption};

	#
	#  Check to see if both the "basic" and "full" options were provided on the
	#  command line.  If so, we drop "basic" and inform the user.
	#
	if ($dataOption eq "basic") {
	    $foundBasic = 1;
	}
	if ($dataOption eq "full") {
	    $foundFull = 1;
	}
    }
}

if ($foundBasic && $foundFull) {
    print "Options basic and full are mutually exclusive.  Defaulting to full.\n";
    push(@badCollectOptions, "basic");
    $collectDataBits &= ~($OBDC_Common::dataCollectValues{$dataOption});
}
    
foreach $dataOption (@badCollectOptions) {
    $collectDataStr =~ s/$dataOption[,]?//;
}

if ($collectDataStr eq "") {
    &ErrorExit("No valid data collection options provided.");    
}

#
# Determine product platform.
#
my ($serverExecutable, $clientExecutable, $adminExecutable, $hardwareInfo,
    $softwareInfo, $initFunction);

SWITCH: {

    if ($platform eq "linux") {
	$home = $ENV{'SMISDIR'} ? $ENV{'SMISDIR'} : "/opt/IBM/smis/tapeagent"; 
	$hardwareInfo = \%OBDC_Linux::Pmf_HW;
	$softwareInfo = \%OBDC_Linux::Pmf_SW;
	$initFunction = \&OBDC_Linux::Pmf_InitPlatform;
	$archiveFunction = \&OBDC_Linux::Pmf_Archive;
	$serverExecutable = "${home}/pegasus/bin/cimserver";
	$clientExecutable = "${home}/client/bin/stfs.o";
	$adminExecutable = "sfscli";
	$pathSeparator = "/";
	last SWITCH;
    }
    else {
	&ErrorExit("Script does not run on, $platform");
    }
}

$topOutputDir = "${home}${pathSeparator}OBDC-Dump";

#
# Determine product functionality 
#
if ( -d "${home}" && 
     -s "${serverExecutable}") {
    $tankRoleStr .= "server";
    $tankRoleBits |= $OBDC_Common::roleValues{"server"};
    unshift(@INC, "${home}${pathSeparator}server${pathSeparator}perl");
}

#
# If tankRoleStr is undefined, then something is wrong with the installation.
#
if ($tankRoleStr eq "") {
    &ErrorExit("Could not determine role for this machine");
}

#
# Create the output directories.  The sequence is to create a directory with the 
# name of the machine, followed by a time-stamp name directory.
#
$_ = `hostname`;
chop;
@machineName = split(/\./, $_);
$outputDir = "${topOutputDir}";
if (! -d $outputDir) {
    mkdir("$outputDir", 0775) || 
	&ErrorExit("Failed to create directory, $outputDir: $!");
}

#
# Setup a time based suffix for the directory we will create.
# Ensure we use two digits for month, day, hour, minute, and seconds.
#
my ($seconds, $minutes, $hour, $monthDay, $month, $year, $weekDay, 
    $yearDay, $isdst, $timeStamp);

($seconds,$minutes,$hour,$monthDay,$month,$year,$weekDay,$yearDay,$isdst) = &OBDC_Common::FormatTimeData(0);
$timeStamp = "$year$month$monthDay"."_$hour$minutes$seconds";
$archiveName = "$archiveName"."-${timeStamp}";

#
# Check for available disk space in output directory.
# (statvfs?)
# TODO

#
# Create directory where all output will be placed for this instance of the 
# script.  Note that the script expects the parent directory to exist, e.g. 
# /opt/IBM/smis/tapeagent/OBDC-Dump.
#
$outputDir = "${topOutputDir}${pathSeparator}${timeStamp}";
mkdir("$outputDir", 0775) || 
    &ErrorExit("Failed to create directory, $outputDir: $!");
print "OBDC Dump output is ${topOutputDir}${pathSeparator}${archiveName}.tgz\n";

#
# Setup standard output and standard error for all messages from this script,
# and the commands it executes, to go to the defined output file.
#
open(ORIGOUT, ">&STDOUT");
open(ORIGERR, ">&STDERR");

open(STDOUT, ">${outputDir}${pathSeparator}${dataFile}") || 
    &ErrorExit("Cannot redirect standard output: $!");
open(STDERR, ">&STDOUT") || 
    &ErrorExit("Cannot redirect standard error: $!");

&$initFunction();

#
# Update the PMF log file with information from this instance.
#
$pmfLogFile = "${outputDir}${pathSeparator}${myname}-${timeStamp}.log";
$_ = ctime(time);
chop;
if (open(PMFLOG, ">>$pmfLogFile")) {
    print PMFLOG "START IBM TapeLibrary SMI-S Agent OBDC V$myversion ($$) $_ \n";
}
else {
    print "Failed to update logfile, $pmfLogFile.\n";
}

print "IBM TapeLibrary SMI-S Agent OBDC ${myversion} Dump -- START -- $_\n\n";
print "Collection Options -- $collectDataStr\n";
 
PrintStartSection("Hardware");
GatherData($hardwareInfo);
PrintEndSection("Hardware");

PrintStartSection("Software");
GatherData($softwareInfo);
PrintEndSection("Software");

$_ = ctime(time);
chop;
print "\nIBM TapeLibrary SMI-S Agent Data Capture -- END -- $_\n";

if (PMFLOG) {
    print PMFLOG "END   ($$) -- $_\n";
    close(PMFLOG);
}

#
# Restore STDOUT and STDERR.
#
close(STDOUT);
close(STDERR);

open(STDOUT, ">&ORIGOUT");
open(STDERR, ">&ORIGERR");

#
# If requested to do so, archive and compress all of the data.
#
if ($timeStamp) {
    print "Archiving information.\n";
    &$archiveFunction($timeStamp);
}

exit 0;
